//=====================================================================
//	Library for controlling the slope of balance line.
//=====================================================================

//---------------------------------------------------------------------
#property copyright 	"Dima S., 2010."
#property link      	"dimascub@mail.com"
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//---------------------------------------------------------------------
enum ControlType
{
    STEP_WITH_HYSTERESISH = 0,				// Step With Hysteresis
    STEP_WITHOUT_HYSTERESIS = 1,				// Step Without Hysteresis
    LINEAR = 2,									// Linear
    NON_LINEAR = 3,								// Nonlinear
};
//---------------------------------------------------------------------
//---------------------------------------------------------------------
enum LotsState
{
	LOTS_NORMAL = 1,								// mode of trading with normal volume
	LOTS_REJECTED = -1,							// mode of trading with lower volume
	LOTS_INTERMEDIATE = 0,						// mode of trading with intermediate volume
};
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//	Operations with work symbol:
//---------------------------------------------------------------------
class TradeSymbol
{
private:
	string		trade_symbol;					// used symbol

private:
	double		min_trade_volume;				// minimum acceptable volume for trade operations
	double		max_trade_volume;				// maximum acceptable volume for trade operations
	double		min_trade_volume_step;		// minimum step of volume change
	double		max_total_volume;				// maximum cumulative volume by a symbol
	double		symbol_point;					// value of one point
	double		symbol_point_multiplier;	// multiplier for 
	double		symbol_tick_size;				// minimum change of price
	int				symbol_digits;				// number of digits after decimal point
	datetime	symbol_time;						// time of coming of last quote
	double		symbol_stop_level;			// minimum level for setting SL ( points )
	double		symbol_spread;					// spread
	double		symbol_bid;
	double		symbol_ask;
 
protected:

public:
	void			RefreshSymbolInfo( );											// update market information of the work symbol
	void			SetTradeSymbol( string _symbol );							// set/change work symbol
	string		GetTradeSymbol( );												// get work symbol
	double		GetMaxTotalLots( );												// get maximum cumulative volume
	double		GetMinTradeVolume( );											// get minimal acceptable volume of trade operations
	double		GetMaxTradeVolume( );											// get maximum acceptable volume of trade operations
	double		GetPoints( double _delta );									// get step price change in points
	double		GetDelta( double _points );									// get change of price as absolute value
	datetime	GetSymbolTime( );														// get time of coming of last quote

public:
	double		NormalizeLots( double _requied_lot );						// get normalized trade volume
	double		NormalizePrice( double _org_price );						// get normalized price with consideration of step of quote change
	double		NormalizeStopLoss( double _stop_loss, long _type );	// get normalized SL
	double		GetStopLoss( double _points, long _type );				// get price value of SL specified in points
	double		GetTakeProfit( double _points, long _type );				// get price value of TP specified in points

public:
	void			TradeSymbol( );				 // constructor
	void			~TradeSymbol( );				 // destructor
};

//---------------------------------------------------------------------
//	Constructor:
//---------------------------------------------------------------------
void
TradeSymbol::TradeSymbol( )
{
	this.trade_symbol = NULL;
}

//---------------------------------------------------------------------
//	Destructor:
//---------------------------------------------------------------------
void
TradeSymbol::~TradeSymbol( )
{
}

//---------------------------------------------------------------------
//	Update market information by work symbol:
//---------------------------------------------------------------------
void
TradeSymbol::RefreshSymbolInfo( )
{
//	If a work symbol is not specified, don't do anything:
	if( this.GetTradeSymbol( ) == NULL )
	{
		return;
	}

//	Calculate parameters that are necessary for normalization of volume:
	this.min_trade_volume = SymbolInfoDouble( this.GetTradeSymbol( ), SYMBOL_VOLUME_MIN );
	this.max_trade_volume = SymbolInfoDouble( this.GetTradeSymbol( ), SYMBOL_VOLUME_MAX );
	this.min_trade_volume_step = SymbolInfoDouble( this.GetTradeSymbol( ), SYMBOL_VOLUME_STEP );

	this.max_total_volume = SymbolInfoDouble( this.GetTradeSymbol( ), SYMBOL_VOLUME_LIMIT );

	this.symbol_point = SymbolInfoDouble( this.GetTradeSymbol( ), SYMBOL_POINT );
	this.symbol_tick_size = SymbolInfoDouble( this.GetTradeSymbol( ), SYMBOL_TRADE_TICK_SIZE );
	this.symbol_digits = ( int )SymbolInfoInteger( this.GetTradeSymbol( ), SYMBOL_DIGITS );

	this.symbol_stop_level = SymbolInfoInteger( this.GetTradeSymbol( ), SYMBOL_TRADE_STOPS_LEVEL );
	this.symbol_spread = SymbolInfoInteger( this.GetTradeSymbol( ), SYMBOL_SPREAD );
	this.symbol_bid = SymbolInfoDouble( this.GetTradeSymbol( ), SYMBOL_BID );
	this.symbol_ask = SymbolInfoDouble( this.GetTradeSymbol( ), SYMBOL_ASK );

}

//---------------------------------------------------------------------
//	Get work symbol:
//---------------------------------------------------------------------
string
TradeSymbol::GetTradeSymbol( )
{
	return( this.trade_symbol );
}

//---------------------------------------------------------------------
//	Set/change work symbol:
//---------------------------------------------------------------------
void
TradeSymbol::SetTradeSymbol( string _symbol )
{
	this.trade_symbol = _symbol;
	this.RefreshSymbolInfo( );
}

//---------------------------------------------------------------------
//	Get maximum cumulative volume:
//---------------------------------------------------------------------
double
TradeSymbol::GetMaxTotalLots( )
{
	return( this.max_total_volume );
}

//---------------------------------------------------------------------
//	Get minimum acceptable volume of trade operations:
//---------------------------------------------------------------------
double
TradeSymbol::GetMinTradeVolume( )
{
	return( this.min_trade_volume );
}

//---------------------------------------------------------------------
//	Get maximum acceptable volume of trade operations:
//---------------------------------------------------------------------
double
TradeSymbol::GetMaxTradeVolume( )
{
	return( this.max_trade_volume );
}

//---------------------------------------------------------------------
//	Get time of coming of last quote: 
//---------------------------------------------------------------------
datetime
TradeSymbol::GetSymbolTime( )
{
	this.symbol_time = ( datetime )SymbolInfoInteger( this.GetTradeSymbol( ), SYMBOL_TIME );
	return( this.symbol_time );
}

//---------------------------------------------------------------------
//	Get change of price in points:
//---------------------------------------------------------------------
double
TradeSymbol::GetPoints( double _delta )
{
	return( NormalizeDouble( _delta / this.symbol_point, 0 ));
}

//---------------------------------------------------------------------
//	Get change of price as absolute value:
//---------------------------------------------------------------------
double
TradeSymbol::GetDelta( double _points )
{
	return( NormalizeDouble( _points * this.symbol_point, this.symbol_digits ));
}

//---------------------------------------------------------------------
//	Get price value of SL:
//---------------------------------------------------------------------
double
TradeSymbol::GetStopLoss( double _points, long _type )
{
	if( _type == ( long )POSITION_TYPE_BUY )
	{
		return( NormalizeStopLoss( this.symbol_bid - _points * this.symbol_point, _type ));
	}
	else if( _type == ( long )POSITION_TYPE_SELL )
	{
		return( NormalizeStopLoss( this.symbol_ask + _points * this.symbol_point, _type ));
	}

	return( 0.0 );
}

//---------------------------------------------------------------------
//	Get price value of TP:
//---------------------------------------------------------------------
double
TradeSymbol::GetTakeProfit( double _points, long _type )
{
	if( _type == ( long )POSITION_TYPE_BUY )
	{
		return( NormalizeDouble( this.symbol_ask + _points * this.symbol_point, this.symbol_digits ));
	}
	else if( _type == ( long )POSITION_TYPE_SELL )
	{
			return( NormalizeDouble( this.symbol_bid - _points * this.symbol_point, this.symbol_digits ));
	}

	return( 0.0 );
}

//---------------------------------------------------------------------
//	Get normalized trade volume:
//---------------------------------------------------------------------
//	- input necessary volume;
//	- get normalized profit as output;
//---------------------------------------------------------------------
double
TradeSymbol::NormalizeLots( double _requied_lots )
{
	double	lots, koeff;
	int			nmbr;

//	If a work symbol is not specified, don't do anything:
	if( this.GetTradeSymbol( ) == NULL )
	{
		return( 0.0 );
	}

	if( this.min_trade_volume_step > 0.0 )
	{
		koeff = 1.0 / this.min_trade_volume_step;
		nmbr = ( int )MathLog10( koeff );
	}
	else
	{
		koeff = 1.0 / this.min_trade_volume;
		nmbr = 2;
	}
	lots = MathFloor( _requied_lots * koeff ) / koeff;

//	Minimum limit of volume:
	if( lots < this.min_trade_volume )
	{
		lots = this.min_trade_volume;
	}

//	Maximum limit of volume:
	if( lots > this.max_trade_volume )
	{
		lots = this.max_trade_volume;
	}

	lots = NormalizeDouble( lots, nmbr );
	return( lots );
}

//---------------------------------------------------------------------
//	Normalize price considering the step of quote change:
//---------------------------------------------------------------------
double
TradeSymbol::NormalizePrice( double _org_price )
{
//	Minimum step of change of quotes in points:
	double	min_price_step = NormalizeDouble( this.symbol_tick_size / this.symbol_point, 0 );

	double	norm_price = NormalizeDouble( NormalizeDouble(( NormalizeDouble( _org_price / this.symbol_point, 0 )) / min_price_step, 0 ) * min_price_step * this.symbol_point, this.symbol_digits );
	return( norm_price );
}

//---------------------------------------------------------------------
//	Check if the price value of SP is correct:
//---------------------------------------------------------------------
//	- returns new price value of SL;
//	- if initial SL is incorrect, take the closest value to the necessary one;
//	- normalizes returned SL in accordance with symbol type;
//---------------------------------------------------------------------
double
TradeSymbol::NormalizeStopLoss( double _stop_loss, long _type )
{
	double	corrected_stop = _stop_loss;
	double	delta = NormalizeDouble(( this.symbol_stop_level + 2.0 * this.symbol_spread ) * this.symbol_point, this.symbol_digits );

	if( _type == ( long )POSITION_TYPE_BUY )
	{
//	Check if we want to set a stop level closer than the one already set (check with a small reserve):
		if( /*MathAbs*/( this.symbol_bid - corrected_stop ) <= delta )
		{
			corrected_stop = this.symbol_bid - delta;			// correct it to be as close as possible to the price
		}
	}
	else if( _type == ( long )POSITION_TYPE_SELL )
	{
//	Check if we want to set a stop level closer than the one already set (check with a small reserve):
		if( /*MathAbs*/( corrected_stop - this.symbol_ask ) <= delta )
		{
			corrected_stop = this.symbol_ask + delta;			// correct it to be as close as possible to the price
		}
	}

	return( this.NormalizePrice( corrected_stop ));
}

//---------------------------------------------------------------------
//	Operations with balance history:
//---------------------------------------------------------------------
class TBalanceHistory
{
private:
	long			current_magic;										// value of "magic number" when accessing the history of deals ( 0 - any number )
	long			current_type;										// type of deals ( -1 - all )
	int				current_limit_history;						// maximum depth of history of deals ( 0 - entire history )
	datetime	monitoring_begin_date;								// date of start of monitoring deals
	int				real_trades;									// number of already performed trades 

public:
	TradeSymbol	trade_symbol;										// operations with work symbol

protected:
//	"Raw" arrays:
	double		org_datetime_array[ ];							// / 
	double		org_result_array[ ];								// trade result

//	Arrays with data grouped by time:
	double		group_datetime_array[ ];						// date/time of trade
	double		group_result_array[ ];							// trade result

	double		last_result_array[ ];							// array for storing the results of last trades ( points on the Y axis )
	double		last_datetime_array[ ];							// array for storing time of last trades ( points on X axis )

private:
	void			SortMasterSlaveArray( double& _m[ ], double& _s[ ] );				// synchronous descending sorting of two arrays

public:
	void			SetTradeSymbol( string _symbol );										// set/change work symbols
	string		GetTradeSymbol( );															// get work symbol
	void			RefreshSymbolInfo( );														// refresh market information by work symbols
	void			SetMonitoringBeginDate( datetime _dt );								// set date of start of monitoring
	datetime	GetMonitoringBeginDate( );														// get date of start of monitoring
	int				GetRealTrades( );															// get amount of last trades
	void			SetFiltrParams( long _magic, long _type = -1, int _limit = 0 );// set parameters of filtration of trades

public:
	int				GetTradeResultsArray( int _max_trades );							// calculate result of last trades

public:
	void			TBalanceHistory( );								// constructor
	void			~TBalanceHistory( );								// destructor
};

//---------------------------------------------------------------------
//	Constructor:
//---------------------------------------------------------------------
void
TBalanceHistory::TBalanceHistory( )
{
	this.current_magic = 0;
	this.current_type = -1;
	this.current_limit_history = 0;
	this.monitoring_begin_date = 0;
}

//---------------------------------------------------------------------
//	Destructor:
//---------------------------------------------------------------------
void
TBalanceHistory::~TBalanceHistory( )
{
}

//---------------------------------------------------------------------
//	Get amount of last trades:
//---------------------------------------------------------------------
int
TBalanceHistory::GetRealTrades( )
{
	return( this.real_trades );
}

//---------------------------------------------------------------------
//	Set/change work symbol:
//---------------------------------------------------------------------
void
TBalanceHistory::SetTradeSymbol( string _symbol )
{
	this.trade_symbol.SetTradeSymbol( _symbol );
}

//---------------------------------------------------------------------
//	Get work symbol:
//---------------------------------------------------------------------
string
TBalanceHistory::GetTradeSymbol( )
{
	return( this.trade_symbol.GetTradeSymbol( ));
}

//---------------------------------------------------------------------
//	Update market information by work symbol:
//---------------------------------------------------------------------
void
TBalanceHistory::RefreshSymbolInfo( )
{
	this.trade_symbol.RefreshSymbolInfo( );
}

//---------------------------------------------------------------------
//	Set date of start of monitoring:
//---------------------------------------------------------------------
void
TBalanceHistory::SetMonitoringBeginDate( datetime _dt )
{
	this.monitoring_begin_date = _dt;
}

//---------------------------------------------------------------------
//	Set parameters of filtration of trades:
//---------------------------------------------------------------------
void
TBalanceHistory::SetFiltrParams( long _magic, long _type, int _limit )
{
	this.current_magic = _magic;
	this.current_type = _type;
	this.current_limit_history = _limit;
}

//---------------------------------------------------------------------
//	Get date of start of monitoring:
//---------------------------------------------------------------------
datetime
TBalanceHistory::GetMonitoringBeginDate( )
{
	return( this.monitoring_begin_date );
}

//---------------------------------------------------------------------
//	Synchronous descending right-to-left sorting of two arrays:
//---------------------------------------------------------------------
void
TBalanceHistory::SortMasterSlaveArray( double& _master[ ], double& _slave[ ] )
{
	int				size = ArraySize( _master );
	if( size != ArraySize( _slave ))
	{
		return;
	}

	double		temp_m = 0.0;
	double		temp_s;

	for( int i = 1;  i <= size; i++ )
	{
		for( int j = 0;  j < size - i;  j++ )
		{
//	If the left element is less than the right one, change their places:
			if( _master[ j ] < _master[ j + 1 ] )
			{
				temp_m = _master[ j ]; 
				_master[ j ] = _master[ j + 1 ];
				_master[ j + 1 ] = temp_m;

				temp_s = _slave[ j ]; 
				_slave[ j ] = _slave[ j + 1 ];
				_slave[ j + 1 ] = temp_s;
			}
		}
	}
}

//---------------------------------------------------------------------
//	Reads the result of last (by time) trades to the arrays:
//---------------------------------------------------------------------
//	- returns amount of actually read trades, but no more than specified;
//---------------------------------------------------------------------
int
TBalanceHistory::GetTradeResultsArray( int _max_trades )
{
	int				index, limit, count;
	long			deal_type, deal_magic, deal_entry;
	int				deal_close_time, current_time;
	ulong			deal_ticket;									// deal ticket
	double		trade_result;
	string		symbol, deal_symbol;

	this.real_trades = 0;

//	Number of trades must be no less than two:
	if( _max_trades < 2 )
	{
		return( 0 );
	}

//	If a work symbol is not specified, don't do anything:
	symbol = this.trade_symbol.GetTradeSymbol( );
	if( symbol == NULL )
	{
		return( 0 );
	}

//	Request the history of deals and orders from the specified time to the current moment:
	if( HistorySelect( this.monitoring_begin_date, TimeCurrent( )) != true )
	{
		return( 0 );
	}

//	Count number of trades:
	count = HistoryDealsTotal( );

//	If there are less trades than necessary in the history, then exit:
	if( count < _max_trades )
	{
		return( 0 );
	}

//	If there are more trades than necessary, then limit them:
	if( this.current_limit_history > 0 && count > this.current_limit_history )
	{
		limit = count - this.current_limit_history;
	}
	else
	{
		limit = 0;
	}

//	If needed, adjust number of dimensions of "raw" arrays by the specified number of trades:
	if(( ArraySize( this.org_datetime_array )) != ( count - limit ))
	{
		ArrayResize( this.org_datetime_array, count - limit );
		ArrayResize( this.org_result_array, count - limit );
	}

//	Fill the "raw" array from the base of trades:
	for( index = count - 1; index >= limit; index-- )
	{
		deal_ticket = HistoryDealGetTicket( index );

//	If they are not closed trades, don't go further:
		deal_entry = HistoryDealGetInteger( deal_ticket, DEAL_ENTRY );
		if( deal_entry != DEAL_ENTRY_OUT )
		{
			continue;
		}

//	Check "magic number" of deal if necessary:
		deal_magic = HistoryDealGetInteger( deal_ticket, DEAL_MAGIC );
		if( this.current_magic != 0 && deal_magic != this.current_magic )
		{
			continue;
		}

//	Check symbol of deal:
		deal_symbol = HistoryDealGetString( deal_ticket, DEAL_SYMBOL );
		if( symbol != deal_symbol )
		{
			continue;
		}
		
//	Check type of deal if necessary:
		deal_type = HistoryDealGetInteger( deal_ticket, DEAL_TYPE );
		if( this.current_type != -1 && deal_type != this.current_type )
		{
			continue;
		}
		else if( this.current_type == -1 && ( deal_type != DEAL_TYPE_BUY && deal_type != DEAL_TYPE_SELL ))
		{
			continue;
		}
		
//	Check time of closing of deal:
		deal_close_time = ( int )HistoryDealGetInteger( deal_ticket, DEAL_TIME );
		if( deal_close_time < ( int )this.monitoring_begin_date )
		{
			continue;
		}

//	Thus, we can read another trade:
		this.org_datetime_array[ this.real_trades ] = ( double )( deal_close_time / 60 );
		this.org_result_array[ this.real_trades ] = HistoryDealGetDouble( deal_ticket, DEAL_PROFIT ) / HistoryDealGetDouble( deal_ticket, DEAL_VOLUME );
		this.real_trades++;
	}

//	If there are less trades than necessary, then exit:
	if( this.real_trades < _max_trades )
	{
		return( 0 );
	}
	count = this.real_trades;

//	Sort the "raw" array by date/time of closing of orders:
	SortMasterSlaveArray( this.org_datetime_array, this.org_result_array );

//	If needed, adjust dimensionality of group arrays with the specified number of points:
	if(( ArraySize( this.group_datetime_array )) != count )
	{
		ArrayResize( this.group_datetime_array, count );
		ArrayResize( this.group_result_array, count );
	}
	ArrayInitialize( this.group_datetime_array, 0.0 );
	ArrayInitialize( this.group_result_array, 0.0 );

//	Fill the output array with grouped information (group by the identity of date/time closing positions):
	this.real_trades = 0;
	for( index = 0; index < count; index++ )
	{
//	Get another trade:
		deal_close_time = ( int )this.org_datetime_array[ index ];
		trade_result = this.org_result_array[ index ];

//	Now check if the same time already exists in the output array:
		current_time = ( int )this.group_datetime_array[ this.real_trades ];
		if( current_time > 0 && ( int )MathAbs( current_time - deal_close_time ) > 0 )
		{
			this.real_trades++;												// move pointer to the next element
			this.group_result_array[ this.real_trades ] = trade_result;
			this.group_datetime_array[ this.real_trades ] = deal_close_time;
		}
		else
		{
			this.group_result_array[ this.real_trades ] += trade_result;
			this.group_datetime_array[ this.real_trades ] = deal_close_time;
		}
	}
	this.real_trades++;														// now this is the number of unique elements

//	If there are less trades than necessary, then exit:
	if( this.real_trades < _max_trades )
	{
		return( 0 );
	}

	if( ArraySize( this.last_result_array ) != _max_trades )
	{
		ArrayResize( this.last_result_array, _max_trades );
		ArrayResize( this.last_datetime_array, _max_trades );
	}

//	Write the accumulated information to output arrays with the reversed indexation:
	for( index = 0; index < _max_trades; index++ )
	{
		this.last_result_array[ _max_trades - 1 - index ] = this.group_result_array[ index ];
		this.last_datetime_array[ _max_trades - 1 - index ] = this.group_datetime_array[ index ];
	}

//	Replace the results of single trades with the growing total amount in the output array:
	for( index = 1; index < _max_trades; index++ )
	{
		this.last_result_array[ index ] += this.last_result_array[ index - 1 ];
	}

	return( _max_trades );
}


//---------------------------------------------------------------------
//	Operations with balance curve:
//---------------------------------------------------------------------
class TBalanceSlope : public TBalanceHistory
{
protected:
	double	current_slope;										// current slope angle of balance curve

private:
	int			slope_count_points;							// number of points (trades) for calculation of slope angle
	
private:
	double	LR_koeff_A, LR_koeff_B;							// coefficients for the equation of the straight-line regression
	double	LR_points_array[ ];								// array of points of the straight-line regression

private:
	void		CalcLR( double& X[ ], double& Y[ ] );		// calculate the equation of the straight-line regression

public:
	void		SetSlopePoints( int _number );				// set number of points for calculation of the straight-line regression
	int			GetSlopePoints( );							// get number of points for calculation of the straight-line regression
	double	CalcSlope( );										//   
	double	GetCurrentSlope( );								// calculate current angle of slope

public:
	void		TBalanceSlope( );								   // constructor
	void		~TBalanceSlope( );							   // destructor
};

//---------------------------------------------------------------------
//	Constructor:
//---------------------------------------------------------------------
void
TBalanceSlope::TBalanceSlope( )
{
	this.slope_count_points = 3;
	this.current_slope = 0.0;
}

//---------------------------------------------------------------------
//	Destructor:
//---------------------------------------------------------------------
void
TBalanceSlope::~TBalanceSlope( )
{
}

//---------------------------------------------------------------------
//	Set number of points for calculation of straight-line regression:
//---------------------------------------------------------------------
void
TBalanceSlope::SetSlopePoints( int _number )
{
	this.slope_count_points = _number;
}

//---------------------------------------------------------------------
//	Get number of points for calculation of straight-line regression:
//---------------------------------------------------------------------
int
TBalanceSlope::GetSlopePoints( )
{
	return( this.slope_count_points );
}

//---------------------------------------------------------------------
//	Get current angle of slope:
//---------------------------------------------------------------------
double
TBalanceSlope::GetCurrentSlope( )
{
	return( this.current_slope );
}

//---------------------------------------------------------------------
//	Calculate the equation of straight-line regression:
//---------------------------------------------------------------------
//	Author: Sergey Privalov aka Prival,  Skype: privalov-sv
//---------------------------------------------------------------------
//	input parameters:
//		X[ ] - array of values of the number series along the X axis;
//		Y[ ] - array of values of the number series along the Y axis;
//---------------------------------------------------------------------
void
TBalanceSlope::CalcLR( double& X[ ], double& Y[ ] )
{
	double		mo_X = 0, mo_Y = 0, var_0 = 0, var_1 = 0;
	int				i;
	int				size = ArraySize( X );
	double		nmb = ( double )size;

//	If the number of points is less than two, the straight line cannot be calculated:
	if( size < 2 )
	{
		return;
	}

	for( i = 0; i < size; i++ )
	{
		mo_X += X[ i ];
		mo_Y += Y[ i ];
	}
	mo_X /= nmb;
	mo_Y /= nmb;

	for( i = 0; i < size; i++ )
	{
		var_0 += ( X[ i ] - mo_X ) * ( Y[ i ] - mo_Y );
		var_1 += ( X[ i ] - mo_X ) * ( X[ i ] - mo_X );
	}

//	Value of the A coefficient:
	if( var_1 != 0.0 )
	{
		this.LR_koeff_A = var_0 / var_1;
	}
	else
	{
		this.LR_koeff_A = 0.0;
	}

//	Value of the B coefficient:
	this.LR_koeff_B = mo_Y - this.LR_koeff_A * mo_X;

//	Fill the array of points which lie on the straight line of regression:
	ArrayResize( this.LR_points_array, size );
	for( i = 0; i < size; i++ )
	{
		this.LR_points_array[ i ] = this.LR_koeff_A * X[ i ] + this.LR_koeff_B;
	}
}

//---------------------------------------------------------------------
//	Calculate slope angle:
//---------------------------------------------------------------------
double
TBalanceSlope::CalcSlope( )
{
//	Get the result of trading from the history of trades:
	int			nmb = this.GetTradeResultsArray( this.slope_count_points );
	if( nmb < this.slope_count_points )
	{
		return( 0.0 );
	}

//	Calculate the regression line on the basis of results of the last trades:
	this.CalcLR( last_datetime_array, last_result_array );
	this.current_slope = this.LR_koeff_A;

	return( this.current_slope );
}


//---------------------------------------------------------------------
//	Managing the slope angle of the balance curve:
//---------------------------------------------------------------------
class TBalanceSlopeControl : public TBalanceSlope
{
private:
	double	min_slope;										// slope angle that corresponds to the mode of volume rejection
	double	max_slope;										// angle of slope that corresponds to the mode of normal volume
	double	centr_slope;									// angle of slope that corresponds to the switching of volume without hysteresis

private:
	ControlType	control_type;								// type of the regulating function

private:
	double	rejected_lots;									// volume in the mode of rejection
	double	normal_lots;									//     
	double	intermed_lots;									// volume in the normal mode

private:
	LotsState		current_lots_state;					// current volume

public:
	void		SetControlType( ControlType _control );				      // set type of the regulating characteristics
	void		SetControlParams( double _min_slope, double _max_slope, double _centr_slope );

public:
	double	CalcTradeLots( double _min_lots, double _max_lots );		// get trade volume

protected:
	double	CalcIntermediateLots( double _min_lots, double _max_lots, double _slope );

public:
	void		TBalanceSlopeControl( );					// constructor
	void		~TBalanceSlopeControl( );					// destructor
};

//---------------------------------------------------------------------
//	Constructor:
//---------------------------------------------------------------------
void
TBalanceSlopeControl::TBalanceSlopeControl( )
{
	this.current_lots_state = LOTS_REJECTED;			// start working in the mode of rejection of work volume
}

//---------------------------------------------------------------------
//	Destructor:
//---------------------------------------------------------------------
void
TBalanceSlopeControl::~TBalanceSlopeControl( )
{
}

//---------------------------------------------------------------------
//	Set type of regulating characteristics
//---------------------------------------------------------------------
void
TBalanceSlopeControl::SetControlType( ControlType _control )
{
	this.control_type = _control;
}

//---------------------------------------------------------------------
//	Set parameters of managing the slope of of the balance curve:
//---------------------------------------------------------------------
void
TBalanceSlopeControl::SetControlParams( double _min_slope, double _max_slope, double _centr_slope )
{
	this.min_slope = _min_slope;							// slope angle that corresponds to the mode of volume rejection
	this.max_slope = _max_slope;							// slope angle that corresponds the mode of normal volume
	this.centr_slope = _centr_slope;						// slope angle that corresponds to switching of volume without hysteresis
}

//---------------------------------------------------------------------
//	Get trade volume:
//---------------------------------------------------------------------
double
TBalanceSlopeControl::CalcTradeLots( double _min_lots, double _max_lots )
{
//	Let's try to calculate the slope of the balance curve:
	double	current_slope = this.CalcSlope( );

//	If the necessary amount of trades is not reached yet,  trade with minimum volume:
	if( this.GetRealTrades( ) < this.GetSlopePoints( ))
	{
		this.current_lots_state = LOTS_REJECTED;
		this.rejected_lots = this.trade_symbol.NormalizeLots( _min_lots );
		return( this.rejected_lots );
	}

//	If the function of regulating is stepped and without hysteresis:
	if( this.control_type == STEP_WITHOUT_HYSTERESIS )
	{
		if( current_slope < this.centr_slope )
		{
			this.current_lots_state = LOTS_REJECTED;
			this.rejected_lots = this.trade_symbol.NormalizeLots( _min_lots );
			return( this.rejected_lots );
		}
		else
		{
			this.current_lots_state = LOTS_NORMAL;
			this.normal_lots = this.trade_symbol.NormalizeLots( _max_lots );
			return( this.normal_lots );
		}
	}

//	If the slope of the regression line for the balance curve is less than acceptable:
	if( current_slope < this.min_slope )
	{
		this.current_lots_state = LOTS_REJECTED;
		this.rejected_lots = this.trade_symbol.NormalizeLots( _min_lots );
		return( this.rejected_lots );
	}

//	If the slope of regression line for the balance curve is greater than specified:
	if( current_slope > this.max_slope )
	{
		this.current_lots_state = LOTS_NORMAL;
		this.normal_lots = this.trade_symbol.NormalizeLots( _max_lots );
		return( this.normal_lots );
	}

//	Slope of the regression line for the balance curve is within specified borders (intermediate state):
	this.current_lots_state = LOTS_INTERMEDIATE;

//	Calculate value of the intermediate volume:
	this.intermed_lots = CalcIntermediateLots( _min_lots, _max_lots, current_slope );
	this.intermed_lots = this.trade_symbol.NormalizeLots( this.intermed_lots );

	return( this.intermed_lots );
}

//---------------------------------------------------------------------
//	Calculation of intermediate volume:
//---------------------------------------------------------------------
double
TBalanceSlopeControl::CalcIntermediateLots( double _min_lots, double _max_lots, double _slope )
{
	double		lots;

//	If the function of regulation is stepped and with hysteresis:
	if( this.control_type == STEP_WITH_HYSTERESISH )
	{
		if( this.current_lots_state == LOTS_REJECTED && _slope > this.min_slope && _slope < this.max_slope )
		{
			lots = _min_lots;
		}
		else if( this.current_lots_state == LOTS_NORMAL && _slope > this.min_slope && _slope < this.max_slope )
		{
			lots = _max_lots;
		}
	}
//	If the function of regulation is linear:
	else if( this.control_type == LINEAR )
	{
		double	a = ( _max_lots - _min_lots ) / ( this.max_slope - this.min_slope );
		double	b = normal_lots - a * this.max_slope;
		lots = a * _slope + b;
	}
//	If the function of regulation is not linear ( not implemented yet ):
	else if( this.control_type == NON_LINEAR )
	{
		lots = _min_lots;
	}
//	If the function of regulation is unknown:
	else
	{
		lots = _min_lots;
	}

	return( lots );
}
//---------------------------------------------------------------------
